@@ -439,6 +439,19 @@ end
       ]])
    end)
 
+   it("detects unreachable code in nested function", function()
+      assert.same({
+         {code = "511", line = 4, column = 7, end_column = 12}
+      }, check[[
+return function()
+   return function()
+      do return end
+      return
+   end
+end
+      ]])
+   end)
+
    it("detects accessing uninitialized variables", function()
       assert.same({
          {code = "113", name = "get", line = 6, column = 8, end_column = 10},
@@ -456,6 +469,25 @@ return a
       ]])
    end)
 
+   it("detects accessing uninitialized variables in nested functions", function()
+      assert.same({
+         {code = "113", name = "get", line = 7, column = 8, end_column = 10},
+         {code = "321", name = "a", line = 7, column = 12, end_column = 12}
+      }, check[[
+return function() return function(...)
+local a
+
+if ... then
+   a = 5
+else
+   a = get(a)
+end
+
+return a
+end end
+      ]])
+   end)
+
    it("does not detect accessing unitialized variables incorrectly in loops", function()
       assert.same({
          {code = "113", name = "get", line = 4, column = 8, end_column = 10}
@@ -505,6 +537,32 @@ repeat until ...
       ]])
    end)
 
+   it("detects empty statements", function()
+      assert.same({
+         {code = "551", line = 1, column = 1, end_column = 1},
+         {code = "541", line = 2, column = 1, end_column = 2},
+         {code = "551", line = 2, column = 8, end_column = 8},
+         {code = "551", line = 4, column = 20, end_column = 20},
+         {code = "551", line = 7, column = 17, end_column = 17}
+      }, check[[
+;
+do end;;
+local foo = "bar";
+foo = foo .. "baz";;
+
+while true do
+   if foo() then;
+      goto fail;
+   elseif foo() then
+      break;
+   end
+end
+
+::fail::
+return foo;
+      ]])
+   end)
+
    it("handles argparse sample", function()
       assert.table(check(io.open("spec/samples/argparse.lua", "rb"):read("*a")))
    end)
