@@ -79,6 +79,7 @@ describe("lexer", function()
       assert.same({token = "/"}, get_token("/ /"))
       assert.same({token = "//"}, get_token("//"))
       assert.same({token = "."}, get_token(".?."))
+      assert.same({token = "."}, get_token("."))
       assert.same({token = ".."}, get_token("..%"))
       assert.same({token = "...", token_value = "..."}, get_token("..."))
       assert.same({token = ":"}, get_token(":.:"))
@@ -135,6 +136,7 @@ bar"]]))
          assert.same({token = "string", token_value = "foo bar"}, get_token([["foo b\97r"]]))
          assert.same({token = "string", token_value = "\1234"}, get_token([["\1234"]]))
          assert.same({line = 1, column = 2, end_column = 5, msg = "invalid decimal escape sequence '\\300'"}, get_error([["\300"]]))
+         assert.same({line = 1, column = 2, end_column = 2, msg = "invalid escape sequence '\\'"}, get_error([["\]]))
       end)
 
       it("parses hexadecimal escape sequences correctly", function()
@@ -165,6 +167,7 @@ bar"]]))
          assert.same({line = 1, column = 2, end_column = 7, msg = "invalid UTF-8 escape sequence '\\u{11u'"}, get_error([["\u{11unrelated"]]))
          assert.same({line = 1, column = 2, end_column = 6, msg = "invalid UTF-8 escape sequence '\\u{11'"}, get_error([["\u{11]]))
          assert.same({line = 1, column = 2, end_column = 5, msg = "invalid UTF-8 escape sequence '\\u{u'"}, get_error([["\u{unrelated}"]]))
+         assert.same({line = 1, column = 2, end_column = 4, msg = "invalid UTF-8 escape sequence '\\u{'"}, get_error([["\u{]]))
       end)
 
       it("detects unknown escape sequences", function()
