@@ -61,7 +61,7 @@ describe("parser", function()
                      {tag = "Number", "1"},
                      {tag = "String", "foo"}
                   }, get_node("return 1, 'foo'"))
-      assert.same({line = 1, column = 10, end_column = 10, msg = "unexpected symbol near <eof>"}, get_error("return 1,"))
+      assert.same({line = 1, column = 10, end_column = 10, msg = "expected expression near <eof>"}, get_error("return 1,"))
    end)
 
    it("parses labels correctly", function()
@@ -74,7 +74,7 @@ describe("parser", function()
    it("parses goto correctly", function()
       assert.same({tag = "Goto", "fail"}, get_node("goto fail"))
       assert.same({line = 1, column = 5, end_column = 5, msg = "expected identifier near <eof>"}, get_error("goto"))
-      assert.same({line = 1, column = 9, end_column = 9, msg = "unexpected symbol near ','"}, get_error("goto foo, bar"))
+      assert.same({line = 1, column = 9, end_column = 9, msg = "expected statement near ','"}, get_error("goto foo, bar"))
    end)
 
    it("parses break correctly", function()
@@ -86,6 +86,8 @@ describe("parser", function()
       assert.same({tag = "Do"}, get_node("do end"))
       assert.same({line = 1, column = 3, end_column = 3, msg = "expected 'end' near <eof>"}, get_error("do"))
       assert.same({line = 1, column = 4, end_column = 8, msg = "expected 'end' near 'until'"}, get_error("do until false"))
+      assert.same({line = 2, column = 1, end_column = 5, msg = "expected 'end' (to close 'do' on line 1) near 'until'"},
+         get_error("do\nuntil false"))
    end)
 
    it("parses while do end correctly", function()
@@ -93,10 +95,12 @@ describe("parser", function()
                      {tag = "True"},
                      {}
                   }, get_node("while true do end"))
-      assert.same({line = 1, column = 6, end_column = 6, msg = "unexpected symbol near <eof>"}, get_error("while"))
+      assert.same({line = 1, column = 6, end_column = 6, msg = "expected condition near <eof>"}, get_error("while"))
       assert.same({line = 1, column = 11, end_column = 11, msg = "expected 'do' near <eof>"}, get_error("while true"))
       assert.same({line = 1, column = 14, end_column = 14, msg = "expected 'end' near <eof>"}, get_error("while true do"))
-      assert.same({line = 1, column = 7, end_column = 8, msg = "unexpected symbol near 'do'"}, get_error("while do end"))
+      assert.same({line = 2, column = 3, end_column = 3, msg = "expected 'end' (to close 'while' on line 1) near <eof>"},
+         get_error("while true\ndo"))
+      assert.same({line = 1, column = 7, end_column = 8, msg = "expected condition near 'do'"}, get_error("while do end"))
       assert.same({line = 1, column = 11, end_column = 11, msg = "expected 'do' near ','"}, get_error("while true, false do end"))
    end)
 
@@ -106,8 +110,10 @@ describe("parser", function()
                      {tag = "True"}
                   }, get_node("repeat until true"))
       assert.same({line = 1, column = 7, end_column = 7, msg = "expected 'until' near <eof>"}, get_error("repeat"))
-      assert.same({line = 1, column = 13, end_column = 13, msg = "unexpected symbol near <eof>"}, get_error("repeat until"))
-      assert.same({line = 1, column = 18, end_column = 18, msg = "unexpected symbol near ','"}, get_error("repeat until true, false"))
+      assert.same({line = 3, column = 1, end_column = 1, msg = "expected 'until' (to close 'repeat' on line 1) near <eof>"},
+         get_error("repeat\n--"))
+      assert.same({line = 1, column = 13, end_column = 13, msg = "expected condition near <eof>"}, get_error("repeat until"))
+      assert.same({line = 1, column = 18, end_column = 18, msg = "expected statement near ','"}, get_error("repeat until true, false"))
    end)
 
    describe("when parsing if", function()
@@ -116,10 +122,12 @@ describe("parser", function()
                         {tag = "True"},
                         {}
                      }, get_node("if true then end"))
-         assert.same({line = 1, column = 3, end_column = 3, msg = "unexpected symbol near <eof>"}, get_error("if"))
+         assert.same({line = 1, column = 3, end_column = 3, msg = "expected condition near <eof>"}, get_error("if"))
          assert.same({line = 1, column = 8, end_column = 8, msg = "expected 'then' near <eof>"}, get_error("if true"))
          assert.same({line = 1, column = 13, end_column = 13, msg = "expected 'end' near <eof>"}, get_error("if true then"))
-         assert.same({line = 1, column = 4, end_column = 7, msg = "unexpected symbol near 'then'"}, get_error("if then end"))
+         assert.same({line = 2, column = 5, end_column = 5, msg = "expected 'end' (to close 'if' on line 1) near <eof>"},
+            get_error("if true\nthen"))
+         assert.same({line = 1, column = 4, end_column = 7, msg = "expected condition near 'then'"}, get_error("if then end"))
          assert.same({line = 1, column = 8, end_column = 8, msg = "expected 'then' near ','"}, get_error("if true, false then end"))
       end)
 
@@ -130,6 +138,8 @@ describe("parser", function()
                         {}
                      }, get_node("if true then else end"))
          assert.same({line = 1, column = 18, end_column = 18, msg = "expected 'end' near <eof>"}, get_error("if true then else"))
+         assert.same({line = 3, column = 1, end_column = 1, msg = "expected 'end' (to close 'else' on line 2) near <eof>"},
+            get_error("if true\nthen else\n"))
          assert.same({line = 1, column = 19, end_column = 22, msg = "expected 'end' near 'else'"}, get_error("if true then else else end"))
       end)
 
@@ -140,8 +150,10 @@ describe("parser", function()
                         {tag = "False"},
                         {}
                      }, get_node("if true then elseif false then end"))
-         assert.same({line = 1, column = 21, end_column = 23, msg = "unexpected symbol near 'end'"}, get_error("if true then elseif end"))
-         assert.same({line = 1, column = 21, end_column = 24, msg = "unexpected symbol near 'then'"}, get_error("if true then elseif then end"))
+         assert.same({line = 1, column = 21, end_column = 23, msg = "expected condition near 'end'"}, get_error("if true then elseif end"))
+         assert.same({line = 1, column = 21, end_column = 24, msg = "expected condition near 'then'"}, get_error("if true then elseif then end"))
+         assert.same({line = 2, column = 5, end_column = 5, msg = "expected 'end' (to close 'elseif' on line 1) near <eof>"},
+            get_error("if true then elseif a\nthen"))
       end)
 
       it("parses if then elseif then else end correctly", function()
@@ -169,6 +181,8 @@ describe("parser", function()
          assert.same({line = 1, column = 7, end_column = 8, msg = "expected '=', ',' or 'in' near '~='"}, get_error("for i ~= 2"))
          assert.same({line = 1, column = 11, end_column = 12, msg = "expected ',' near 'do'"}, get_error("for i = 2 do end"))
          assert.same({line = 1, column = 15, end_column = 15, msg = "expected 'end' near <eof>"}, get_error("for i=1, #t do"))
+         assert.same({line = 2, column = 4, end_column = 4, msg = "expected 'end' (to close 'for' on line 1) near <eof>"},
+            get_error("for i=1, #t do\na()"))
          assert.same({line = 1, column = 5, end_column = 5, msg = "expected identifier near '('"}, get_error("for (i)=1, #t do end"))
          assert.same({line = 1, column = 5, end_column = 5, msg = "expected identifier near '3'"}, get_error("for 3=1, #t do end"))
       end)
@@ -202,7 +216,7 @@ describe("parser", function()
                         {}
                      }, get_node("for i, j in t, 'foo' do end"))
          assert.same({line = 1, column = 5, end_column = 6, msg = "expected identifier near 'in'"}, get_error("for in foo do end"))
-         assert.same({line = 1, column = 10, end_column = 11, msg = "unexpected symbol near 'do'"}, get_error("for i in do end"))
+         assert.same({line = 1, column = 10, end_column = 11, msg = "expected expression near 'do'"}, get_error("for i in do end"))
       end)
    end)
 
@@ -218,10 +232,12 @@ describe("parser", function()
          assert.same({line = 1, column = 11, end_column = 11, msg = "expected '(' near <eof>"}, get_error("function a"))
          assert.same({line = 1, column = 12, end_column = 12, msg = "expected argument near <eof>"}, get_error("function a("))
          assert.same({line = 1, column = 13, end_column = 13, msg = "expected 'end' near <eof>"}, get_error("function a()"))
+         assert.same({line = 2, column = 2, end_column = 2, msg = "expected 'end' (to close 'function' on line 1) near <eof>"},
+            get_error("function a(\n)"))
          assert.same({line = 1, column = 10, end_column = 10, msg = "expected identifier near '('"}, get_error("function (a)()"))
          assert.same({line = 1, column = 9, end_column = 9, msg = "expected identifier near '('"}, get_error("function() end"))
          assert.same({line = 1, column = 11, end_column = 11, msg = "expected '(' near 'a'"}, get_error("(function a() end)"))
-         assert.same({line = 1, column = 18, end_column = 18, msg = "unexpected symbol near ')'"}, get_error("function a() end()"))
+         assert.same({line = 1, column = 18, end_column = 18, msg = "expected expression near ')'"}, get_error("function a() end()"))
       end)
 
       it("parses simple function with arguments correctly", function()
@@ -245,6 +261,8 @@ describe("parser", function()
                      }, get_node("function a(b, ...) end"))
          assert.same({line = 1, column = 15, end_column = 15, msg = "expected argument near ')'"}, get_error("function a(b, ) end"))
          assert.same({line = 1, column = 13, end_column = 13, msg = "expected ')' near '.'"}, get_error("function a(b.c) end"))
+         assert.same({line = 2, column = 2, end_column = 2, msg = "expected ')' (to close '(' on line 1) near '.'"},
+            get_error("function a(\nb.c) end"))
          assert.same({line = 1, column = 12, end_column = 12, msg = "expected argument near '('"}, get_error("function a((b)) end"))
          assert.same({line = 1, column = 15, end_column = 15, msg = "expected ')' near ','"}, get_error("function a(..., ...) end"))
       end)
@@ -302,8 +320,8 @@ describe("parser", function()
                      }, get_node("local a, b"))
          assert.same({line = 1, column = 6, end_column = 6, msg = "expected identifier near <eof>"}, get_error("local"))
          assert.same({line = 1, column = 9, end_column = 9, msg = "expected identifier near <eof>"}, get_error("local a,"))
-         assert.same({line = 1, column = 8, end_column = 8, msg = "unexpected symbol near '.'"}, get_error("local a.b"))
-         assert.same({line = 1, column = 8, end_column = 8, msg = "unexpected symbol near '['"}, get_error("local a[b]"))
+         assert.same({line = 1, column = 8, end_column = 8, msg = "expected statement near '.'"}, get_error("local a.b"))
+         assert.same({line = 1, column = 8, end_column = 8, msg = "expected statement near '['"}, get_error("local a[b]"))
          assert.same({line = 1, column = 7, end_column = 7, msg = "expected identifier near '('"}, get_error("local (a)"))
       end)
 
@@ -322,10 +340,10 @@ describe("parser", function()
                            {tag = "Id", "d"}
                         }
                      }, get_node("local a, b = c, d"))
-         assert.same({line = 1, column = 11, end_column = 11, msg = "unexpected symbol near <eof>"}, get_error("local a = "))
-         assert.same({line = 1, column = 13, end_column = 13, msg = "unexpected symbol near <eof>"}, get_error("local a = b,"))
-         assert.same({line = 1, column = 8, end_column = 8, msg = "unexpected symbol near '.'"}, get_error("local a.b = c"))
-         assert.same({line = 1, column = 8, end_column = 8, msg = "unexpected symbol near '['"}, get_error("local a[b] = c"))
+         assert.same({line = 1, column = 11, end_column = 11, msg = "expected expression near <eof>"}, get_error("local a = "))
+         assert.same({line = 1, column = 13, end_column = 13, msg = "expected expression near <eof>"}, get_error("local a = b,"))
+         assert.same({line = 1, column = 8, end_column = 8, msg = "expected statement near '.'"}, get_error("local a.b = c"))
+         assert.same({line = 1, column = 8, end_column = 8, msg = "expected statement near '['"}, get_error("local a[b] = c"))
          assert.same({line = 1, column = 10, end_column = 10, msg = "expected identifier near '('"}, get_error("local a, (b) = c"))
       end)
 
@@ -375,10 +393,10 @@ describe("parser", function()
                         }
                      }, get_node("(f():g())[9] = d"))
          assert.same({line = 1, column = 2, end_column = 2, msg = "expected '=' near <eof>"}, get_error("a"))
-         assert.same({line = 1, column = 5, end_column = 5, msg = "unexpected symbol near <eof>"}, get_error("a = "))
-         assert.same({line = 1, column = 5, end_column = 5, msg = "unexpected symbol near '='"}, get_error("a() = b"))
-         assert.same({line = 1, column = 5, end_column = 5, msg = "syntax error near '='"}, get_error("(a) = b"))
-         assert.same({line = 1, column = 1, end_column = 1, msg = "unexpected symbol near '1'"}, get_error("1 = b"))
+         assert.same({line = 1, column = 5, end_column = 5, msg = "expected expression near <eof>"}, get_error("a = "))
+         assert.same({line = 1, column = 5, end_column = 5, msg = "expected statement near '='"}, get_error("a() = b"))
+         assert.same({line = 1, column = 1, end_column = 1, msg = "expected statement near '('"}, get_error("(a) = b"))
+         assert.same({line = 1, column = 1, end_column = 1, msg = "expected statement near '1'"}, get_error("1 = b"))
       end)
 
       it("parses multi assignment correctly", function()
@@ -391,11 +409,11 @@ describe("parser", function()
                         }
                      }, get_node("a, b = c, d"))
          assert.same({line = 1, column = 5, end_column = 5, msg = "expected '=' near <eof>"}, get_error("a, b"))
-         assert.same({line = 1, column = 4, end_column = 4, msg = "unexpected symbol near '='"}, get_error("a, = b"))
-         assert.same({line = 1, column = 8, end_column = 8, msg = "unexpected symbol near <eof>"}, get_error("a, b = "))
-         assert.same({line = 1, column = 10, end_column = 10, msg = "unexpected symbol near <eof>"}, get_error("a, b = c,"))
-         assert.same({line = 1, column = 8, end_column = 8, msg = "syntax error near '='"}, get_error("a, b() = c"))
-         assert.same({line = 1, column = 8, end_column = 8, msg = "syntax error near '='"}, get_error("a, (b) = c"))
+         assert.same({line = 1, column = 4, end_column = 4, msg = "expected identifier or field near '='"}, get_error("a, = b"))
+         assert.same({line = 1, column = 8, end_column = 8, msg = "expected expression near <eof>"}, get_error("a, b = "))
+         assert.same({line = 1, column = 10, end_column = 10, msg = "expected expression near <eof>"}, get_error("a, b = c,"))
+         assert.same({line = 1, column = 8, end_column = 8, msg = "expected call or indexing near '='"}, get_error("a, b() = c"))
+         assert.same({line = 1, column = 4, end_column = 4, msg = "expected identifier or field near '('"}, get_error("a, (b) = c"))
       end)
    end)
 
@@ -431,10 +449,13 @@ describe("parser", function()
                            {tag = "Id", "b"}
                         }
                      }, get_node("(a)(b)()"))
-         assert.same({line = 1, column = 2, end_column = 2, msg = "unexpected symbol near ')'"}, get_error("()()"))
-         assert.same({line = 1, column = 3, end_column = 3, msg = "unexpected symbol near <eof>"}, get_error("a("))
-         assert.same({line = 1, column = 1, end_column = 1, msg = "unexpected symbol near '1'"}, get_error("1()"))
-         assert.same({line = 1, column = 1, end_column = 5, msg = "unexpected symbol near ''foo''"}, get_error("'foo'()"))
+         assert.same({line = 1, column = 2, end_column = 2, msg = "expected expression near ')'"}, get_error("()()"))
+         assert.same({line = 1, column = 3, end_column = 3, msg = "expected expression near <eof>"}, get_error("a("))
+         assert.same({line = 1, column = 4, end_column = 4, msg = "expected ')' near <eof>"}, get_error("a(b"))
+         assert.same({line = 2, column = 2, end_column = 2, msg = "expected ')' (to close '(' on line 1) near <eof>"},
+            get_error("a(\nb"))
+         assert.same({line = 1, column = 1, end_column = 1, msg = "expected statement near '1'"}, get_error("1()"))
+         assert.same({line = 1, column = 1, end_column = 5, msg = "expected statement near ''foo''"}, get_error("'foo'()"))
          assert.same({line = 1, column = 9, end_column = 9, msg = "expected identifier near '('"}, get_error("function() end ()"))
       end)
 
@@ -475,8 +496,8 @@ describe("parser", function()
                            {tag = "String", "b"}
                         }, {tag = "String", "c"}
                      }, get_node("a:b():c()"))
-         assert.same({line = 1, column = 1, end_column = 1, msg = "unexpected symbol near '1'"}, get_error("1:b()"))
-         assert.same({line = 1, column = 1, end_column = 2, msg = "unexpected symbol near ''''"}, get_error("'':a()"))
+         assert.same({line = 1, column = 1, end_column = 1, msg = "expected statement near '1'"}, get_error("1:b()"))
+         assert.same({line = 1, column = 1, end_column = 2, msg = "expected statement near ''''"}, get_error("'':a()"))
          assert.same({line = 1, column = 9, end_column = 9, msg = "expected identifier near '('"}, get_error("function()end:b()"))
          assert.same({line = 1, column = 4, end_column = 4, msg = "expected method arguments near ':'"}, get_error("a:b:c()"))
          assert.same({line = 1, column = 3, end_column = 3, msg = "expected identifier near <eof>"}, get_error("a:"))
@@ -526,10 +547,16 @@ describe("parser", function()
                         {tag = "Id", "b"},
                         {tag = "Id", "c"}
                      }, get_expr("{a; b, c;}"))
-         assert.same({line = 1, column = 9, end_column = 9, msg = "unexpected symbol near ';'"}, get_error("return {;}"))
-         assert.same({line = 1, column = 9, end_column = 9, msg = "unexpected symbol near <eof>"}, get_error("return {"))
-         assert.same({line = 1, column = 11, end_column = 11, msg = "unexpected symbol near ','"}, get_error("return {a,,}"))
-         assert.same({line = 1, column = 13, end_column = 13, msg = "unexpected symbol near <eof>"}, get_error("return {a = "))
+         assert.same({line = 1, column = 9, end_column = 9, msg = "expected expression near ';'"}, get_error("return {;}"))
+         assert.same({line = 1, column = 9, end_column = 9, msg = "expected expression near <eof>"}, get_error("return {"))
+         assert.same({line = 1, column = 11, end_column = 13, msg = "expected '}' near 'end'"}, get_error("return {a end"))
+         assert.same({line = 2, column = 1, end_column = 3, msg = "expected '}' (to close '{' on line 1) near 'end'"},
+            get_error("return {a\nend"))
+         assert.same({line = 1, column = 11, end_column = 11, msg = "expected ']' near <eof>"}, get_error("return {[a"))
+         assert.same({line = 2, column = 2, end_column = 2, msg = "expected ']' (to close '[' on line 1) near <eof>"},
+            get_error("return {[\na"))
+         assert.same({line = 1, column = 11, end_column = 11, msg = "expected expression near ','"}, get_error("return {a,,}"))
+         assert.same({line = 1, column = 13, end_column = 13, msg = "expected expression near <eof>"}, get_error("return {a = "))
       end)
 
       it("wraps last element in table constructors in parens when needed", function()
@@ -694,7 +721,7 @@ describe("parser", function()
       end)
 
       it("does not allow statements after return", function()
-         assert.same({line = 1, column = 8, end_column = 12, msg = "unexpected symbol near 'break'"}, get_error("return break"))
+         assert.same({line = 1, column = 8, end_column = 12, msg = "expected expression near 'break'"}, get_error("return break"))
          assert.same({line = 1, column = 9, end_column = 13, msg = "expected end of block near 'break'"}, get_error("return; break"))
          assert.same({line = 1, column = 8, end_column = 8, msg = "expected end of block near ';'"}, get_error("return;;"))
          assert.same({line = 1, column = 10, end_column = 14, msg = "expected end of block near 'break'"}, get_error("return 1 break"))
